package edu.umd.cs.voting.model;

import java.util.ArrayList;
import java.util.Iterator;

import org.jdom.Attribute;
import org.jdom.DataConversionException;
import org.jdom.Element;

public class Item {

	protected Page page;
	protected String title;
	protected String subtitle;
	protected String question;
	protected ArrayList candidates;
	protected ArrayList selectedCandidates;
	protected int maxSelectedCandidates;	
	public float height;
	protected boolean pageBreak;
	
	public Item(Page page, Element itemElement) {
		try {
			this.page = page;
			
			title = itemElement.getAttributeValue("title");
			subtitle = itemElement.getAttributeValue("subtitle");
			maxSelectedCandidates = itemElement.getAttribute("votefor").getIntValue();
			Attribute pageBreakAttribute = itemElement.getAttribute("pageBreak");
			if (pageBreakAttribute != null) {
				pageBreak = pageBreakAttribute.getBooleanValue();
			} else {
				pageBreak = false;
			}
			Attribute heightAttribute = itemElement.getAttribute("height");
			if (heightAttribute != null) {
				height = heightAttribute.getFloatValue(); 
			}
			
			selectedCandidates = new ArrayList();
			
			if (title != null) title = title.replaceAll("@br", "\n");
			if (subtitle != null) subtitle = subtitle.replaceAll("@br", "\n");
			
			Element questionElement = itemElement.getChild("question");
			if (questionElement != null) {
				question = questionElement.getTextTrim();
			}

			candidates = new ArrayList();
			
			Iterator i = itemElement.getChild("candidates").getChildren().iterator();
			while (i.hasNext()) {
				Element candiateElement = (Element) i.next();
				Candidate candidate = new Candidate(this, candiateElement);
				
				// all non-null parties must be declared so that party column layout is possible.
//				if (candidate.getParty() != null && !page.ballot.parties.contains(candidate.getParty())) {
	//				throw new RuntimeException("all candiate parties must be declared in ballot. " + candidate.getParty() + " undeclared");
		//		}
				candidates.add(candidate);
			}
		} catch (DataConversionException e) {
			e.printStackTrace();
		}
	}
	
	public String getTitle() {
		return title;
	}

	public String getSubtitle() {
		return subtitle;
	}

	public String getQuestion() {
		return question;
	}
	
	public ArrayList getCandidates() {
		return candidates;
	}
	
	public int getMaxSelectedCandidates() {
		return maxSelectedCandidates;
	}
	
	public boolean getPageBreak() {
		return pageBreak;
	}
	
	public int getNumCurrentlySelectedCandidates() {
		return selectedCandidates.size();
/*		int result = 0;
		
		for (int i = 0; i < candidates.size(); i++) {
			Candidate each = (Candidate) candidates.get(i);
			if (each.isSelected()) {
				result++;
			}
		}
		return result;*/
	}
	
	public boolean canSelectAnother() {
		return selectedCandidates.size() < maxSelectedCandidates;
	}	
	
	public void select(Candidate candidate) {
		selectedCandidates.add(candidate);
		candidate.setIsSelected(true);
	}
	
	public Candidate getFirstSelected() {
		return (Candidate) selectedCandidates.get(selectedCandidates.size() - 1);
	}
	
	public void unselect(Candidate candidate) {
		if (selectedCandidates.contains(candidate)) {
			selectedCandidates.remove(candidate);
			candidate.setIsSelected(false);
		}
	}
	
	public void reset() {
		for (int i = 0; i < candidates.size(); i++) {
			Candidate each = (Candidate) candidates.get(i);
			each.setIsSelected(false);
		}
		selectedCandidates.clear();
	}
}
