package edu.umd.cs.voting.view;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.geom.GeneralPath;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.SwingUtilities;

import edu.umd.cs.piccolo.PCanvas;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolo.util.PBounds;
import edu.umd.cs.piccolo.util.PDebug;
import edu.umd.cs.piccolox.event.PNotification;
import edu.umd.cs.piccolox.event.PNotificationCenter;
import edu.umd.cs.voting.Controller;

public class NavigationPanelNode extends PNode {

	public static ButtonNode PREVIOUS_BUTTON;
	public static ButtonNode NEXT_BUTTON;
	public static ButtonNode REVIEW_AND_SUBMIT_BUTTON;

	protected ArrayList buttons;	
	protected ButtonNode selectedButton;
	protected PPath selectedButtonDecoration;

	public NavigationPanelNode(PCanvas navigationCanvas, ArrayList thumbnails) {
		
		setPaint(Color.WHITE);

		double xOffset = ButtonPageNode.MARGIN;
		double yOffset = ButtonPageNode.MARGIN;

		buttons = thumbnails;
		
		buttons.add(0, new ButtonNode("Overview", null) {
			public PNode getTargetNode() {
				return Controller.CONTROLLER.getBallotNode();
			}
		});
		
		PREVIOUS_BUTTON = new ButtonNode("Previous", null) {
			public PNode getTargetNode() {
				return Controller.CONTROLLER.getBallotNode().previousFocus();
			}
			public boolean isSelectable() {
				return false;
			}
		};
		PREVIOUS_BUTTON.setPaint(ButtonNode.NEXT_PREVIOUSE_BUTTON_COLOR);
		PREVIOUS_BUTTON.buttonLabel.setFont(ButtonNode.NEXT_PREVIOUSE_BUTTON_LABEL_FONT);
		PREVIOUS_BUTTON.positionLabel();
		
		NEXT_BUTTON = new ButtonNode("Next", null) {
			public PNode getTargetNode() {
				return Controller.CONTROLLER.getBallotNode().nextFocus();				
			}
			public boolean isSelectable() {
				return false;
			}
		};
		NEXT_BUTTON.setPaint(ButtonNode.NEXT_PREVIOUSE_BUTTON_COLOR);
		NEXT_BUTTON.buttonLabel.setFont(ButtonNode.NEXT_PREVIOUSE_BUTTON_LABEL_FONT);
		NEXT_BUTTON.positionLabel();

		buttons.add(2, new ButtonSpacer());
		
		buttons.add(buttons.size(), new ButtonSpacer());

		buttons.add(buttons.size(), PREVIOUS_BUTTON);
		buttons.add(buttons.size(), NEXT_BUTTON);

		buttons.add(buttons.size(), new ButtonSpacer());

		REVIEW_AND_SUBMIT_BUTTON = new ButtonNode("Review and Cast Ballot", null) {
			public PNode getTargetNode() {
				if (Controller.IS_REVIEW_MODE) {
					Controller.CONTROLLER.submitVote();
					return null;
				} else {
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							Controller.CONTROLLER.setReviewMode(true);			
						}
					});
					return Controller.CONTROLLER.getBallotNode();
				}
			}
			public boolean isSelectable() {
				return false;
			}
		};
		
		REVIEW_AND_SUBMIT_BUTTON.setPaint(ButtonNode.REVIEW_AND_SUBMIT_BUTTON_COLOR);
		REVIEW_AND_SUBMIT_BUTTON.buttonLabel.setFont(ButtonNode.REVIEW_AND_SUBMIT_BUTTON_LABEL_FONT);
		REVIEW_AND_SUBMIT_BUTTON.positionLabel();
		
		buttons.add(REVIEW_AND_SUBMIT_BUTTON);

		double totalWidth = xOffset;
		Iterator i = thumbnails.iterator();
		while (i.hasNext()) {
			PNode each = (PNode) i.next();
			totalWidth += each.getWidth() + ButtonPageNode.MARGIN;
		}
		
		double extraWidth = navigationCanvas.getCamera().getWidth() - totalWidth;

		i = thumbnails.iterator();
		while (i.hasNext()) {
			PNode each = (PNode) i.next();
			if (each instanceof ButtonSpacer) {
				each.setWidth(each.getWidth() + (extraWidth / 3));
			}
		}
				
		i = thumbnails.iterator();
		while (i.hasNext()) {
			PNode each = (PNode) i.next();
			each.setOffset(xOffset - each.getX(), yOffset);
			xOffset += each.getWidth() + ButtonPageNode.MARGIN;
			addChild(each);
		}

		double PADDING = 0;
		getUnionOfChildrenBounds(getBoundsReference());
		PBounds b = getBounds();
		b.inset(-(PADDING + ButtonPageNode.MARGIN), -(PADDING + ButtonPageNode.MARGIN));
		setBounds(b);
					
		navigationCanvas.getLayer().addChild(this);
		navigationCanvas.getCamera().animateViewToCenterBounds(getGlobalFullBounds(), true, 0);
		navigationCanvas.getCamera().translateView(0, 100);
		navigationCanvas.getCamera().animateViewToPanToBounds(getGlobalFullBounds(), 0);

		PBounds pageButtonBounds = new PBounds();
		i = thumbnails.iterator();
		while (i.hasNext()) {
			PNode each = (PNode) i.next();
			if (each instanceof ButtonPageNode) {
				ButtonPageNode buttonPageNode = (ButtonPageNode) each;
				if (buttonPageNode.questionCount > 0) {
					pageButtonBounds.add(each.getFullBoundsReference());					
				}
			}
		}

		PText text = new PText("Number of Selections Made:");
		addChild(text);
		text.centerFullBoundsOnPoint(pageButtonBounds.getCenterX(), pageButtonBounds.getY() - text.getHeight() - 2);
		
		PNotificationCenter.defaultCenter().addListener(this, "focusNodeChanged", Controller.FOCUS_CHANGED_NOTIFICATION, null);
		
		addInputEventListener(new PBasicInputEventHandler() {
			public void mousePressed(PInputEvent event) {
				PNode n = event.getPickedNode();
				if (n instanceof ButtonNode) {
					ButtonNode buttonNode = (ButtonNode) n;
					if (buttonNode.isEnabled()) {
						n.setPaint(((Color)n.getPaint()).darker());
					}
				}
			}
			
			public void mouseReleased(PInputEvent event) {
				PNode n = event.getPickedNode();
				if (n instanceof ButtonNode) {
					ButtonNode buttonNode = (ButtonNode) n;
					
					if (buttonNode.isEnabled()) {
						n.setPaint(((Color)n.getPaint()).brighter());
						if (buttonNode.isSelectable()) {
							setSelectedButton(buttonNode);
						}
						PNode target = buttonNode.getTargetNode();
						if (target != null)
							Controller.CONTROLLER.setFocusNode(target);
					}
				}
			}
		});
	}

	public ButtonNode getButtonForTargetNode(PNode node) {
		Iterator i = buttons.iterator();
		while (i.hasNext()) {
			ButtonNode each = (ButtonNode) i.next();
			if (each.isSelectable() && each.getTargetNode() == node) {
				return each;
			}
		}
		return null;
	}

	public void focusNodeChanged(PNotification notification) {
		Controller controller = Controller.CONTROLLER;
		PNode n = controller.getFocusNode();

		if (n instanceof BallotNode) {
			setSelectedButton(getButtonForTargetNode(n));
		} else if (n instanceof PageNode) {
			setSelectedButton(getButtonForTargetNode((PageNode)n));
		} else if (n instanceof ItemNode) {
			setSelectedButton(getButtonForTargetNode((PageNode)n.getParent()));
		}
		
		BallotNode ballotNode = controller.getBallotNode();
		if (ballotNode.previousFocus() == controller.getFocusNode()) {
			PREVIOUS_BUTTON.setEnabled(false);
		} else {
			PREVIOUS_BUTTON.setEnabled(true);
		}

		if (ballotNode.nextFocus() == controller.getFocusNode()) {
			NEXT_BUTTON.setEnabled(false);
		} else {
			NEXT_BUTTON.setEnabled(true);
		}
	}
	
	public void setSelectedButton(ButtonNode newSelectedButton) {
		if (selectedButton != newSelectedButton) {
			if (selectedButton != null) {
				selectedButton.unselect();
				selectedButtonDecoration.removeFromParent();
				selectedButtonDecoration = null;
			}
			selectedButton = newSelectedButton;
			if (selectedButton != null) {
				selectedButton.select();

				PBounds b = selectedButton.getFullBounds();
				b.inset(-ButtonPageNode.MARGIN / 2, -ButtonPageNode.MARGIN / 2);
				selectedButtonDecoration = PPath.createRectangle((float) b.x, (float) b.y, (float) b.width, (float) b.height);

				GeneralPath path = new GeneralPath();
				path.moveTo((float) (b.x + (b.width / 2)), (float) (b.y + b.height));
				path.lineTo((float) (b.x + (b.width / 2)) + 10f, (float) (b.y + b.height) + 15f);
				path.lineTo((float) (b.x + (b.width / 2)) - 10f, (float) (b.y + b.height) + 15f);
				path.lineTo((float) (b.x + (b.width / 2)), (float) (b.y + b.height));
				path.closePath();
				PPath arrowPath = new PPath(path);
				arrowPath.setPaint(Color.BLACK);
				selectedButtonDecoration.addChild(arrowPath);

				selectedButtonDecoration.setPaint(null);
				selectedButtonDecoration.setStroke(new BasicStroke(2));

				addChild(selectedButtonDecoration);
			}
		}
	}	
}
