package edu.umd.cs.voting.view;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Stroke;

import org.jdom.Element;

import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolo.util.PPaintContext;
import edu.umd.cs.voting.Controller;

public class ButtonNode extends PNode {
	
	public static Font BUTTON_LABEL_FONT = new Font("Helvetica", Font.PLAIN, 16);
	public static Color BUTTON_COLOR = Color.LIGHT_GRAY;
	public static Color DISABLED_TEXT_COLOR = Color.LIGHT_GRAY;
	public static Color EDGE_COLOR = Color.GRAY;

	public static Font NEXT_PREVIOUSE_BUTTON_LABEL_FONT = new Font("Helvetica", Font.PLAIN, 16);
	public static Color NEXT_PREVIOUSE_BUTTON_COLOR = Color.LIGHT_GRAY;

	public static Font SUBMIT_BUTTON_LABEL_FONT = new Font("Helvetica", Font.PLAIN, 16);
	public static Color SUBMIT_BUTTON_COLOR = Color.LIGHT_GRAY;

	public static Font REVIEW_AND_SUBMIT_BUTTON_LABEL_FONT = new Font("Helvetica", Font.PLAIN, 16);
	public static Color REVIEW_AND_SUBMIT_BUTTON_COLOR = Color.LIGHT_GRAY;
	public static Color REVIEW_AND_SUBMIT_BUTTON_REVIEW_MODE_COLOR = Color.LIGHT_GRAY;

	public static double MARGIN = 20;
	public static Stroke EDGE_STROKE = new BasicStroke(1f);

	public PText buttonLabel;
	public boolean isSelected;
	public boolean isEnabled = true;
	public PNode targetNode;
	
	public ButtonNode(String label, PNode targetNode) {
		setBounds(0, 0, 80, 80);
		buttonLabel = new PText(label);
		buttonLabel.setConstrainWidthToTextWidth(false);
		buttonLabel.setFont(BUTTON_LABEL_FONT);
		buttonLabel.setCenteredText(true);
		buttonLabel.setWidth(getWidth());
		buttonLabel.setPickable(false);
		addChild(buttonLabel);
		this.targetNode = targetNode;
		setPaint(BUTTON_COLOR);
				
		positionLabel();
	}
	
	public void positionLabel() {
		buttonLabel.centerBoundsOnPoint(getBoundsReference().getCenterX(), getBoundsReference().getCenterY());		
	}

	public void addChild(int index, PNode child) {
		super.addChild(index, child);
	}
	
	public void setLable(String lable) {
		buttonLabel.setText(lable);
		positionLabel();		
	}
	
	public boolean isSelectable() {
		return true;
	}
		
	public boolean isEnabled() {
		return isEnabled;
	}
	
	public void select() {
		isSelected = true;
		invalidatePaint();
		setEnabled(false);
	}

	public void unselect() {
		isSelected = false;
		invalidatePaint();
		setEnabled(true);
	}
	
	public void setEnabled(boolean enabled) {
		isEnabled = enabled;
		if (isEnabled) {
			buttonLabel.setTextPaint(Color.BLACK);
		} else {
			buttonLabel.setTextPaint(DISABLED_TEXT_COLOR);
		}
	}

	public PNode getTargetNode() {
		return targetNode;
	}

	protected void paintAfterChildren(PPaintContext paintContext) {
		super.paintAfterChildren(paintContext);

		Graphics2D g2 = paintContext.getGraphics();
		g2.setStroke(EDGE_STROKE);
		g2.setPaint(EDGE_COLOR);
		g2.draw3DRect((int)getX(), (int)getY(), (int)getWidth(), (int)getHeight(), true);
		//g2.draw(getBoundsReference());
	}

	public static void loadStyleInformation(Element style) {
		ButtonNode.BUTTON_LABEL_FONT = Controller.readFontStyle(style.getChild("label"));
		ButtonNode.BUTTON_COLOR = Controller.readColorStyle(style.getAttributeValue("buttoncolor"));
		ButtonNode.EDGE_COLOR = Controller.readColorStyle(style.getAttributeValue("edgecolor"));
	}
	
	public static void loadNextPreviousStyleInformation(Element style) {
		ButtonNode.NEXT_PREVIOUSE_BUTTON_LABEL_FONT = Controller.readFontStyle(style.getChild("label"));
		ButtonNode.NEXT_PREVIOUSE_BUTTON_COLOR = Controller.readColorStyle(style.getAttributeValue("buttoncolor"));
	}

/*	public static void loadSubmitStyleInformation(Element style) {
		ButtonNode.SUBMIT_BUTTON_LABEL_FONT = Controller.readFontStyle(style.getChild("label"));
		ButtonNode.SUBMIT_BUTTON_COLOR = Controller.readColorStyle(style.getAttributeValue("buttoncolor"));
	}
	*/
	public static void loadReviewAndSubmitStyleInformation(Element style) {
		ButtonNode.REVIEW_AND_SUBMIT_BUTTON_LABEL_FONT = Controller.readFontStyle(style.getChild("label"));
		ButtonNode.REVIEW_AND_SUBMIT_BUTTON_COLOR = Controller.readColorStyle(style.getAttributeValue("buttoncolor"));
		ButtonNode.REVIEW_AND_SUBMIT_BUTTON_REVIEW_MODE_COLOR = Controller.readColorStyle(style.getAttributeValue("reviewModeColor"));	
	}
}
